@@ -1,12 +1,14 @@
 package Statistics::Regression;
 
-$VERSION = '0.50';
-my $DATE = "2007/04/04";
-my $MNAME= "Statistics::Regression";
+$VERSION = '0.52';
+my $DATE = "2007/07/07";
+my $MNAME= "$0::Statistics::Regression";
 
 use strict;
 use warnings FATAL => qw{ uninitialized };
 
+use Carp;
+
 ################################################################
 =pod
 
@@ -14,15 +16,13 @@ use warnings FATAL => qw{ uninitialized };
 
   Regression.pm - weighted linear regression package (line+plane fitting)
 
+
 =head1 SYNOPSIS
 
   use Statistics::Regression;
 
   # Create regression object
-  my $reg = Statistics::Regression->new( 
-    3, "sample regression", 
-    [ "const", "someX", "someY" ] 
-  );
+  my $reg = Statistics::Regression->new( "sample regression", [ "const", "someX", "someY" ] );
 
   # Add data points
   $reg->include( 2.0, [ 1.0, 3.0, -1.0 ] );
@@ -30,9 +30,15 @@ use warnings FATAL => qw{ uninitialized };
   $reg->include( 20.0, [ 1.0, 31.0, 0.0 ] );
   $reg->include( 15.0, [ 1.0, 11.0, 2.0 ] );
 
-Please note that you must provide the constant.
+or
 
-  # Print the result
+  my %d;
+  $d{const} = 1.0; $d{someX}= 5.0; $d{someY}= 2.0; $d{ignored}="anything else";
+  $reg->include( 3.0, \%d );  # names are picked off the Regression specification
+
+Please note that *you* must provide the constant if you want one.
+
+  # Finally, print the result
   $reg->print();
 
 This prints the following:
@@ -48,7 +54,29 @@ This prints the following:
   R^2= 0.808, N= 4
   ****************************************************************
 
-Or, use the subroutines to do the work yourself:
+
+
+The hash input method has the advantage that you can now just
+fill the observation hashes with all your variables, and use the
+same code to run regression, changing the regression
+specification at one and only one spot (the new() invokation).
+You do not need to change the inputs in the include() statement.
+For example,
+
+  my @obs;  ## a global variable.  observations are like: %oneobs= %{$obs[1]};
+
+  sub run_regression {
+    my $reg = Statistics::Regression->new( $_[0], $_[2] );
+    foreach my $obshashptr (@obs) { $reg->include( $_[1], $_[3] ); }
+    $reg->print();
+  }
+
+  run_regression("bivariate regression",  $obshashptr->{someY}, [ "const", "someX" ] );
+  run_regression("trivariate regression",  $obshashptr->{someY}, [ "const", "someX", "someZ" ] );
+
+
+
+Of course, you can use the subroutines to do the printing work yourself:
 
   my @theta  = $reg->theta();
   my @se     = $reg->standarderrors();
@@ -64,6 +92,7 @@ In addition, there are some other helper routines, and a
 subroutine linearcombination_variance().  If you don't know what
 this is, don't use it.
 
+
 =head1 BACKGROUND WARNING
 
 You should have an understanding of OLS regressions if you want
@@ -139,10 +168,23 @@ missing variables (coded as 'NaN' in Regression.pm).
 =for comment
 pod2html -noindex -title "perl weighted least squares regression package" Regression.pm > Regression.html
 
+
 =head1 VERSION
 
 0.50, 2007/04/04
 
+
+=head1 RECENT CHANGES
+
+2007/04/04:  Added Coefficient Standard Errors
+
+2007/07/01:  Added self-test use (if invoked as perl Regression.pm)
+	     at the end.  cleaned up some print sprintf.
+             changed syntax on new() to eliminate passing K.
+
+2007/07/07:  allowed passing hash with names to include().
+
+
 =head1 AUTHOR
 
 Naturally, Gentleman invented this algorithm.  It was adaptated
@@ -158,11 +200,6 @@ This module is released for free public use under a GPL license.
 
 =cut
 
-=head1 RECENT CHANGES
-
-2007/04/04:  Added Coefficient Standard Errors
-
-=cut
 
 ################################################################
 #### let's start with handling of missing data ("nan" or "NaN")
@@ -171,27 +208,37 @@ use constant TINY => 1e-8;
 my $nan= "NaN";
 
 sub isNaN { 
-  if ($_[0] !~ /[0-9nan]/) { die "$MNAME: definitely not a number in NaN: '$_[0]'"; }
+  if ($_[0] !~ /[0-9nan]/) { confess "$MNAME:isNaN: definitely not a number in NaN: '$_[0]'"; }
   return ($_[0]=~ /NaN/i) || ($_[0] != $_[0]);
 }
 
 
 ################################################################
-### my $reg = Statistics::Regression->new($n, $name, \@var_names)
+### my $reg = Statistics::Regression->new($regname, \@var_names)
 ###
 ### Receives the number of variables on each observations (i.e.,
 ### an integer) and returns the blessed data structure as a
 ### Statistics::Regression object. Also takes an optional name
 ### for this regression to remember, as well as a reference to a
 ### k*1 array of names for the X coefficients.
+###
+### I have now made it mandatory to give some names.
+###
 ################################################################
 sub new {
-  my $classname= shift(@_);
-  my $K= shift(@_); # the number of variables
-  my $regname= shift(@_) || "with no name";
+  my $classname= shift;  (!ref($classname)) or confess "$MNAME:new: bad class call to new ($classname).\n";
+  my $regname= shift || "no-name";
+  my $xnameptr= shift;
 
-  if (!defined($K)) { die "$MNAME: Regression->new needs at least one argument for the number of variables"; }
-  if ($K<=1) { die "$MNAME: Cannot run a regression without at least two variables."; }
+  (defined($regname)) or confess "$MNAME:new: bad name in for regression.  no undef allowed.\n";
+  (!ref($regname)) or confess "$MNAME:new: bad name in for regression.\n";
+  (defined($xnameptr)) or confess "$MNAME:new: You must provide variable names, because this tells me the number of columns.  no undef allowed.\n";
+  (ref($xnameptr) eq "ARRAY") or confess "$MNAME:new: bad xnames for regression. Must be pointer.\n";
+
+  my $K= (@{$xnameptr});
+
+  if (!defined($K)) { confess "$MNAME:new: cannot determine the number of variables"; }
+  if ($K<=1) { confess "$MNAME:new: Cannot run a regression without at least two variables."; }
 
   sub zerovec {
     my @rv;
@@ -202,7 +249,7 @@ sub new {
   bless {
 	 k => $K,
 	 regname => $regname,
-	 xnames => shift(@_),
+	 xnames => $xnameptr,
 
 	 # constantly updated
 	 n => 0,
@@ -234,25 +281,56 @@ sub new {
 ### inclusion with a weight of -1 can be used to delete an
 ### observation.
 ###
+### The error checking and transfer of arguments is clutzy, but
+### works.  if I had POSIX assured, I could do better number
+### checking.  right now, I don't do any.
+###
 ### Returns the number of observations so far included.
 ################################################################
 sub include {
-  my $this = shift();
-  my $yelement= shift();
-  my $xrow= shift();
-  my $weight= shift() || 1.0;
+  my $this = shift;
+  my $yelement= shift;
+  my $xin= shift;
+  my $weight= shift || 1.0;
+
+  # modest input checking;
+  (ref($this)) or confess "$MNAME:include: bad class call to include.\n";
+  (defined($yelement)) or confess "$MNAME:include: bad call for y to include.  no undef allowed.\n";
+  (!ref($yelement)) or confess "$MNAME:include: bad call for y to include.  need scalar.\n";
+  (defined($xin)) or confess "$MNAME:include: bad call for x to include.  no undef allowed.\n";
+  (ref($xin)) or confess "$MNAME:include: bad call for x to include. need reference.\n";
+  (!ref($weight)) or confess "$MNAME:include: bad call for weight to include. need scalar.\n";
+
 
   # omit observations with missing observations;
-  (defined($yelement)) or die "$MNAME: you must give a y value (predictor).";
-  (isNaN($yelement)) and return $this->{n};  # ignore
+  (defined($yelement)) or confess "$MNAME:include: you must give a y value (predictor).";
+  (isNaN($yelement)) and return $this->{n};  # ignore this observation;
+  ## should check for number, not string
+
+
+  # check and transfer the X vector
+  my @xrow;
+  if (ref($xin) eq "ARRAY") { @xrow= @{$xin}; }
+  else {
+    my $xctr=0;
+    foreach my $nm (@{$this->{xnames}}) {
+      (defined($xin->{$nm})) or confess "$MNAME:include: Variable '$nm' needs to be set in hash.\n";
+      $xrow[$xctr]= $xin->{$nm};
+      ++$xctr;
+    }
+  }
 
   my @xcopy;
   for (my $i=1; $i<=$this->{k}; ++$i) { 
-    (defined($xrow->[$i-1])) or die "$MNAME: Internal Error: an x cannot be undef.  use NaN for missing.";
-    (isNaN($xrow->[$i-1])) and return $this->{n};
-    $xcopy[$i]= $xrow->[$i-1];
+    (defined($xrow[$i-1]))
+      or confess "$MNAME:include: Internal Error: at N=".($this->{n}).", the x[".($i-1)."] is undef.  use NaN for missing.";
+    (isNaN($xrow[$i-1])) and return $this->{n};
+    $xcopy[$i]= $xrow[$i-1];
+    ## should check for number, not string
   }
 
+  ################ now comes the real routine
+
   $this->{syy}+= ($weight*($yelement*$yelement));
   $this->{sy}+= ($weight*($yelement));
   if ($weight>=0.0) { ++$this->{n}; } else { --$this->{n}; }
@@ -271,7 +349,7 @@ sub include {
       $weight*=($cbar);
       $this->{d}->[$i]=$dprimei;
       my $nextr=int( (($i-1)*( (2.0*$this->{k}-$i))/2.0+1) );
-      if (!($nextr<=$this->{rbarsize}) ) { die "$MNAME: Internal Error 2"; }
+      if (!($nextr<=$this->{rbarsize}) ) { confess "$MNAME:include: Internal Error 2"; }
       my $xk;
       for (my $kc=$i+1;$kc<=$this->{k};++$kc) {
 	$xk=$xcopy[$kc]; $xcopy[$kc]=$xk-$xi*$this->{rbar}->[$nextr];
@@ -305,49 +383,50 @@ sub include {
 ################################################################
 
 sub rsq {
-  my $this= shift();
+  my $this= shift;
   return $this->{rsq}= 1.0- $this->{sse} / $this->sst();
 }
 
 sub adjrsq {
-  my $this= shift();
+  my $this= shift;
   return $this->{adjrsq}= 1.0- (1.0- $this->rsq())*($this->{n}-1)/($this->{n} - $this->{k});
 }
 
 sub sigmasq {
-  my $this= shift();
+  my $this= shift;
   return $this->{sigmasq}= ($this->{n}<=$this->{k}) ? "Inf" : ($this->{sse}/($this->{n} - $this->{k}));
 }
 
 sub ybar {
-  my $this= shift();
+  my $this= shift;
   return $this->{ybar}= $this->{sy}/$this->{wghtn};
 }
 
 sub sst {
-  my $this= shift();
+  my $this= shift;
   return $this->{sst}= ($this->{syy} - $this->{wghtn}*($this->ybar())**2);
 }
 
 sub k {
-  my $this= shift();
+  my $this= shift;
   return $this->{k};
 }
 sub n {
-  my $this= shift();
+  my $this= shift;
   return $this->{n};
 }
 
 
 
 ################################################################
-###  $reg->print()
+###  $reg->print()  [no arguments!]
 ###
 ### prints the estimated coefficients, and R^2 and N. For an
 ### example see the Synopsis.
 ################################################################
 sub print {
-  my $this= $_[0];
+  my $this= shift;
+
   print "****************************************************************\n";
   print "Regression '$this->{regname}'\n";
   print "****************************************************************\n";
@@ -355,17 +434,17 @@ sub print {
   my $theta= $this->theta();
   my @standarderrors= $this->standarderrors();
 
-  print sprintf("%-15s\t%12s\t%12s\t%7s\n", "Name", "Theta", "StdErr", "T-stat");
+  printf "%-15s\t%12s\t%12s\t%7s\n", "Name", "Theta", "StdErr", "T-stat";
   for (my $i=0; $i< $this->k(); ++$i) {
     my $name= "[$i".(defined($this->{xnames}->[$i]) ? "='$this->{xnames}->[$i]'":"")."]";
-    print sprintf("%-15s", $name)."\t";
-    print sprintf("%12.4f", $theta->[$i])."\t";
-    print sprintf("%12.4f", $standarderrors[$i])."\t";
-    print sprintf("%7.2f", ($theta->[$i]/$standarderrors[$i]));
-    print "\n";
+    printf "%-15s\t", $name;
+    printf "%12.4f\t", $theta->[$i];
+    printf "%12.4f\t", $standarderrors[$i];
+    printf "%7.2f", ($theta->[$i]/$standarderrors[$i]);
+    printf "\n";
   }
 
-  print "\nR^2= ".sprintf("%.3f", $this->rsq()).", N= ".$this->n()."\n";
+  print "\nR^2= ".sprintf("%.3f", $this->rsq()).", N= ".$this->n().", K= ".$this->k()."\n";
   print "****************************************************************\n";
 }
 
@@ -380,7 +459,7 @@ sub print {
 ### coefficients.
 ################################################################
 sub theta {
-  my $this= shift();
+  my $this= shift;
 
   if (defined($this->{theta})) { 
     return wantarray ? @{$this->{theta}} : $this->{theta}; 
@@ -390,7 +469,7 @@ sub theta {
   for (my $i=($this->{k}); $i>=1; --$i) {
     $this->{theta}->[$i]= $this->{thetabar}->[$i];
     my $nextr= int (($i-1)*((2.0*$this->{k}-$i))/2.0+1);
-    if (!($nextr<=$this->{rbarsize})) { die "$MNAME: Internal Error 3"; }
+    if (!($nextr<=$this->{rbarsize})) { confess "$MNAME:theta: Internal Error 3"; }
     for (my $kc=$i+1;$kc<=$this->{k};++$kc) {
       $this->{theta}->[$i]-=($this->{rbar}->[$nextr]*$this->{theta}->[$kc]);
       ++$nextr;
@@ -462,14 +541,14 @@ sub theta {
 my $debug=0;
 
 sub standarderrors {
-  my $this= shift();
-  our $K= $this->{k};  # global, nasty, quick; gets set and forgotten
+  my $this= shift;
+  my $K= $this->{k};  # convenience
 
   our @u;
   sub ui {
     if ($debug) {
-      ($_[0]<1)||($_[0]>$K) and die "$MNAME: bad index 0 $_[0]\n";
-      ($_[1]<1)||($_[1]>$K) and die "$MNAME: bad index 1 $_[0]\n";
+      ($_[0]<1)||($_[0]>$K) and confess "$MNAME:standarderrors: bad index 0 $_[0]\n";
+      ($_[1]<1)||($_[1]>$K) and confess "$MNAME:standarderrors: bad index 1 $_[0]\n";
     }
     return (($K*($_[0]-1))+($_[1]-1));
   }
@@ -483,7 +562,7 @@ sub standarderrors {
   sub add2u { return $u[ui($_[0], $_[1])]+= $_[2]; }
   sub mult2u { return $u[ui($_[0], $_[1])]*= $_[2]; }
 
-  (defined($K)) or die "$MNAME: Internal Error: I forgot the number of variables.\n";
+  (defined($K)) or confess "$MNAME:standarderrors: Internal Error: I forgot the number of variables.\n";
   if ($debug) {
     print "The Start Matrix is:\n";
     for (my $i=1; $i<=$K; ++$i) {
@@ -507,7 +586,7 @@ sub standarderrors {
 
   # now a real member routine;
   sub rbr {
-    my $this= shift();
+    my $this= shift;
     return ($_[0] == $_[1]) ? 1 : ( ($_[0]>$_[1]) ? 0 : ($this->{rbar}[rbrindex($_[0],$_[1])]));
   }
 
@@ -542,7 +621,7 @@ sub standarderrors {
 	    for (my $i=0; $i<($K**2); ++$i) { $u[$i]= "NaN"; }
 	    return undef;
 	  }
-	  else { die "$MNAME: I cannot compute the theta-covariance matrix for variable $j ".($this->{d}[$j])."\n"; }
+	  else { confess "$MNAME:standarderrors: I cannot compute the theta-covariance matrix for variable $j ".($this->{d}[$j])."\n"; }
 	}
       }
       else { mult2u($i,$j, sqrt(1.0/$this->{d}[$j])); }
@@ -602,13 +681,14 @@ sub standarderrors {
 }
 
 
+################################
 sub linearcombination_variance {
-  my $this= shift();
-  our $K= $this->{k};
+  my $this= shift;
+  my $K= $this->{k};  # convenience
 
   my @linear= @_;
 
-  ($#linear+1 == $K) or die "$MNAME: linearcombination_variance:".
+  ($#linear+1 == $K) or confess "$MNAME:linearcombination_variance: ".
     "Sorry, you must give a vector of length $K, not ".($#linear+1)."\n";
 
   my @allback= $this->standarderrors();  # compute everything we need;
@@ -628,7 +708,7 @@ sub linearcombination_variance {
 
 
 ################################################################
-### sub dump() is used for debugging.
+### sub dump() was used internally for debugging.
 ################################################################
 sub dump {
   my $this= $_[0];
@@ -670,8 +750,28 @@ sub dump {
   print "****************************************************************\n";
 }
 
+################################################################
+### The Test Program.  Invoke as "perl Regression.pm".
+################################################################
+
+
+if ($0 eq "Regression.pm") {
+
+  # Create regression object
+  my $reg = Statistics::Regression->new( "sample regression", [ "const", "someX", "someY" ] );
+
+  # Add data points
+  $reg->include( 2.0, [ 1.0, 3.0, -1.0 ] );
+  $reg->include( 1.0, [ 1.0, 5.0, 2.0 ] );
+  $reg->include( 20.0, [ 1.0, 31.0, 0.0 ] );
+
+  my %inhash= ( const => 1.0, someX => 11.0, someY => 2.0, ignored => "ignored" );
+  $reg->include( 15.0, \%inhash );
+  # $reg->include( 15.0, [ 1.0, 11.0, 2.0 ] );
+
+  # Print the result
+  $reg->print();
+}
 
 
 1;
-
-# vim: sw=2 sts=2
